/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: popfastlistpenclass.c,v 1.6 2006/09/18 23:17:29 fab Exp $
 */

#include "ambient.h"

/* public */
#include <graphics/rpattr.h>
#include <proto/cybergraphics.h>
#include <proto/graphics.h>

/* private */
#include "mui_func.h"
#include "gfx_pen.h"


#define PFLPTEXT "Example"

struct Data {
	struct MUI_PenSpec fgspec;
	struct MUI_PenSpec bgspec;
	ULONG fgcolor;
	ULONG bgcolor;
	ULONG update_colors;
	ULONG textlen;
	APTR win;
	STRPTR wintitle;
};

/* XXX: implement MUIA_Window_Title.. */

static void doset(APTR obj, struct Data *data, struct TagItem *tags)
{
	FORTAG(tags)
	{
		case MA_Popfastlistpen_FgSpec:
			memcpy(&data->fgspec, (UBYTE *)tag->ti_Data, sizeof(data->fgspec));
			data->update_colors = TRUE;
			break;

		case MA_Popfastlistpen_BgSpec:
			memcpy(&data->bgspec, (UBYTE *)tag->ti_Data, sizeof(data->bgspec));
			data->update_colors = TRUE;
			break;
	
		case MA_Popfastlistpen_WinObj:
			data->win = (APTR)tag->ti_Data;
			break;

		case MUIA_Window_Title:
			data->wintitle = (STRPTR)tag->ti_Data;
			break;
	}
	NEXTTAG
}


DEFNEW
{
	struct Data *data;

	obj = DoSuperNew(cl, obj,
		ButtonFrame,
		MUIA_Background, MUII_ButtonBack,
		MUIA_InnerLeft, 0,
		MUIA_InnerRight, 0,
		MUIA_InnerTop, 0,
		MUIA_InnerBottom, 0,
		MUIA_InputMode, MUIV_InputMode_RelVerify,
		MUIA_FillArea, TRUE,
		MUIA_CustomBackfill, TRUE,
		//MUIA_Font, MUIV_Font_List, /* XXX: set fonts properly, do that from outside ? or not.. hm */
		TAG_MORE, INITTAGS
	);

	if (!obj)
	{
		return (NULL);
	}

	data = INST_DATA(cl, obj);

	doset(obj, data, INITTAGS);

	DoMethod(obj, MUIM_Notify, MUIA_Pressed, FALSE,
		MUIV_Notify_Self, 1, MM_Popfastlistpen_Open
	);

	return ((ULONG)obj);
}


DEFDISP
{
	GETDATA;

	if (data->win)
	{
		DoMethod(app, OM_REMMEMBER, data->win);
		MUI_DisposeObject(data->win);
	}
	return (DOSUPER);
}


DEFSET
{
	GETDATA;

	doset(obj, data, INITTAGS);

	if (data->update_colors)
	{
		DoMethod(obj, MM_Popfastlistpen_UpdateColors, TRUE);
	}

	return (DOSUPER);
}


DEFGET
{
	GETDATA;

	switch (msg->opg_AttrID)
	{
		case MA_Popfastlistpen_FgSpec:
			*msg->opg_Storage = (ULONG)&data->fgspec;
			return (TRUE);

		case MA_Popfastlistpen_BgSpec:
			*msg->opg_Storage = (ULONG)&data->bgspec;
			return (TRUE);
	}
	return (DOSUPER);
}


DEFMMETHOD(Setup)
{
	GETDATA;
	LONG rc = DOSUPER;

	if (data->update_colors)
	{
		DoMethod(obj, MM_Popfastlistpen_UpdateColors, FALSE);
	}

	SetFont(_rp(obj), _font(obj));
	data->textlen = TextLength(_rp(obj), PFLPTEXT, strlen(PFLPTEXT)); /* XXX: we should add a method to update the fonts and use that */

	return (rc);
}


DEFMMETHOD(AskMinMax)
{
	GETDATA;
	ULONG rc;

	rc = DOSUPER;

	msg->MinMaxInfo->MinWidth += data->textlen + 4;
	msg->MinMaxInfo->MinHeight += _font(obj)->tf_YSize + 6; /* XXX: experiment.. */

	msg->MinMaxInfo->DefWidth += data->textlen + 4;
	msg->MinMaxInfo->DefHeight += _font(obj)->tf_YSize + 6; /* XXX: experiment.. */

	msg->MinMaxInfo->MaxWidth = MUI_MAXMAX;
	msg->MinMaxInfo->MaxHeight = MUI_MAXMAX;

	return (rc);
}


DEFMMETHOD(Backfill)
{
	GETDATA;

	SetRPAttrs(_rp(obj),
		RPTAG_PenMode, FALSE,
		RPTAG_FgColor, data->bgcolor,
		TAG_DONE
	);

	RectFill(_rp(obj), msg->left, msg->top, msg->right, msg->bottom);

	SetRPAttrs(_rp(obj),
		RPTAG_PenMode, TRUE,
		TAG_DONE
	);

	return (TRUE);
}


DEFSMETHOD(Popfastlistpen_UpdateColors)
{
	GETDATA;

	data->fgcolor = gfx_get_penspec_value(muiRenderInfo(obj), &data->fgspec);
	data->bgcolor = gfx_get_penspec_value(muiRenderInfo(obj), &data->bgspec);

	if (msg->refresh)
	{
		MUI_Redraw(obj, MADF_DRAWOBJECT);
	}
	data->update_colors = FALSE;

	return (0);
}


DEFMMETHOD(Draw)
{
	GETDATA;
	ULONG rc;

	rc = DOSUPER;

	/* XXX: check the flags? what for.. */

	if(msg->flags & MADF_DRAWOBJECT)
	{
		SetRPAttrs(_rp(obj),
			RPTAG_PenMode, FALSE,
			RPTAG_FgColor, data->fgcolor,
			TAG_DONE
		);

		SetFont(_rp(obj), _font(obj)); /* XXX: put that in a proper place */
		Move(_rp(obj), _mleft(obj) + ((_mwidth(obj) - data->textlen) / 2), _mtop(obj) + (_font(obj)->tf_Baseline + (_mheight(obj) - _font(obj)->tf_Baseline) / 2 - 1));
		Text(_rp(obj), PFLPTEXT, strlen(PFLPTEXT)); /* XXX: remove that strlen() somehow.. */

		SetRPAttrs(_rp(obj),
			RPTAG_PenMode, TRUE,
			TAG_DONE
		);
	}

	return (rc);
}


DEFTMETHOD(Popfastlistpen_Open)
{
	GETDATA;

	if (!data->win)
	{
		if ((data->win = NewObject(getpopfastlistwinclass(), NULL,
			data->wintitle ? MUIA_Window_Title : TAG_IGNORE, data->wintitle,
			MUIA_Window_LeftEdge, _window(obj)->LeftEdge + _left(obj),
			MUIA_Window_TopEdge, _window(obj)->TopEdge + _bottom(obj) + 1,
			MA_Popfastlistwin_Parent, obj,
			MA_Popfastlistwin_FgSpec, &data->fgspec,
			MA_Popfastlistwin_BgSpec, &data->bgspec,
			TAG_DONE)))
		{
			DoMethod(app, OM_ADDMEMBER, data->win);
			set(data->win, MUIA_Window_Open, TRUE);
			/* XXX: check if it opened.. */
		}
		else
		{
			errormsg(ERR_NOMEM);
		}
	}
	else
	{
		set(data->win, MUIA_Window_Open, TRUE);
	}
	return (0);
}


BEGINMTABLE
DECNEW
DECDISPOSE
DECSET
DECGET
DECMMETHOD(Setup)
DECMMETHOD(AskMinMax)
DECMMETHOD(Backfill)
DECMMETHOD(Draw)
DECSMETHOD(Popfastlistpen_UpdateColors)
DECTMETHOD(Popfastlistpen_Open)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Area, popfastlistpenclass)
