/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2001-2005 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: popfastlistwinclass.c,v 1.5 2006/09/18 23:17:30 fab Exp $
 */

#include "ambient.h"

/* public */

/* private */
#include "ambient_cat.h"
#include "methodstack.h"
#include "mui_func.h"


struct Data {
	APTR parent;
	APTR pp_fg;
	APTR pp_bg;
	struct MUI_PenSpec *fgspec; /* pointer to the parent's ones */
	struct MUI_PenSpec *bgspec;
};


DEFNEW
{
	struct Data *data;
	APTR pp_fg, pp_bg, bt_ok, bt_cancel;

	obj = DoSuperNew(cl, obj,
		MUIA_Window_ScreenTitle, screentitle,
		MUIA_Window_ShowIconify, FALSE,
		MUIA_Window_ShowPrefs, FALSE,
		MUIA_Window_ShowJump, FALSE,
		MUIA_Window_ShowAbout, FALSE,
		MUIA_Window_ShowPopup, FALSE,
		MUIA_Window_ShowSnapshot, FALSE,
		WindowContents, VGroup,
			Child, ColGroup(2),
				Child, SLabel1("Foreground:"),
				Child, pp_fg = PoppenObject,
					InnerSpacing(0, 0),
					MUIA_Window_Title, "Ambient  Foreground", /* XXX: holy fuck.. we have to compute those.. */
					MUIA_CycleChain, 1,
				End,
				
				Child, SLabel2("Background:"),
				Child, pp_bg = PoppenObject,
					InnerSpacing(0, 0),
					MUIA_Window_Title, "Ambient  Background", /* XXX: holy fuck.. we have to compute those.. */
					MUIA_CycleChain, 1,
					//MUIA_Disabled, TRUE,
				End,

			End,

			Child, HGroup,
				MUIA_Group_SameSize, TRUE,
				Child, bt_ok = button(MSG_OK, 0),
				Child, HSpace(0),
				Child, HSpace(0),
				Child, bt_cancel = button(MSG_CANCEL, 0),
			End,
		End,
		TAG_MORE, INITTAGS
	);

	if (!obj)
	{
		return (0);
	}

	data = INST_DATA(cl, obj);
	data->pp_fg = pp_fg;
	data->pp_bg = pp_bg;

	FORTAG(INITTAGS)
	{
		case MA_Popfastlistwin_Parent:
			data->parent = (APTR)tag->ti_Data;
			break;

		case MA_Popfastlistwin_FgSpec:
			data->fgspec = (struct MUI_PenSpec *)tag->ti_Data;
			set(data->pp_fg, MUIA_Pendisplay_Spec, data->fgspec);
			break;
	
		case MA_Popfastlistwin_BgSpec:
			data->bgspec = (struct MUI_PenSpec *)tag->ti_Data;
			set(data->pp_bg, MUIA_Pendisplay_Spec, data->bgspec);
			break;
	}
	NEXTTAG

	DoMethod(obj, MUIM_Notify, MUIA_Window_CloseRequest, TRUE,
		obj, 2, MM_Popfastlistwin_Close, MV_Popfastlistwin_Close_Cancel
	);

	DoMethod(bt_cancel, MUIM_Notify, MUIA_Pressed, FALSE,
		obj, 2, MM_Popfastlistwin_Close, MV_Popfastlistwin_Close_Cancel
	);

	DoMethod(bt_ok, MUIM_Notify, MUIA_Pressed, FALSE,
		obj, 2, MM_Popfastlistwin_Close, MV_Popfastlistwin_Close_Ok
	);

	return ((ULONG)obj);
}


DEFGET
{
	switch (msg->opg_AttrID)
	{
		case MA_Window_ID:
			*msg->opg_Storage = 0;
			return (TRUE);

		case MA_Window_Path:
			*msg->opg_Storage = MV_Window_Path_None;
			return (TRUE);

		case MA_Window_Type:
			*msg->opg_Storage = MV_Window_Type_Popfastlist;
			return (TRUE);
	}
	return (DOSUPER);
}


DEFSMETHOD(Popfastlistwin_Close)
{
	GETDATA;

	set(obj, MUIA_Window_Open, FALSE);

	if (msg->val == MV_Popfastlistwin_Close_Ok)
	{
		memcpy(data->fgspec, (UBYTE *)getv(data->pp_fg, MUIA_Pendisplay_Spec), sizeof(*data->fgspec));
		memcpy(data->bgspec, (UBYTE *)getv(data->pp_bg, MUIA_Pendisplay_Spec), sizeof(*data->bgspec));
	
		if (data->parent)
		{
			DoMethod(data->parent, MM_Popfastlistpen_UpdateColors, TRUE);
		}
	}
	if (data->parent)
	{
		set(data->parent, MA_Popfastlistpen_WinObj, FALSE);
	}
	methodstack_push(_app(obj), 2, MM_Application_DisposeWindow, obj);

	return (0);
}


BEGINMTABLE
DECNEW
DECGET
DECSMETHOD(Popfastlistwin_Close)
ENDMTABLE

DECSUBCLASS_NC(MUIC_Window, popfastlistwinclass)
